'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.pricingPageUrl = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; /**
                                                                                                                                                                                                                                                                   * Created by tdzl2003 on 2/13/16.
                                                                                                                                                                                                                                                                   */

exports.question = question;
exports.translateOptions = translateOptions;
exports.getRNVersion = getRNVersion;
exports.getApkInfo = getApkInfo;
exports.getIpaInfo = getIpaInfo;
exports.saveToLocal = saveToLocal;
exports.printVersionCommand = printVersionCommand;

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _os = require('os');

var _os2 = _interopRequireDefault(_os);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const pkg = require('../../package.json');
const AppInfoParser = require('app-info-parser');

var read = require('read');

function question(query, password) {
  if (NO_INTERACTIVE) {
    return Promise.resolve('');
  }
  return new Promise((resolve, reject) => read({
    prompt: query,
    silent: password,
    replace: password ? '*' : undefined
  }, (err, result) => err ? reject(err) : resolve(result)));
}

function translateOptions(options) {
  const ret = {};
  for (let key in options) {
    const v = options[key];
    if (typeof v === 'string') {
      ret[key] = v.replace(/\$\{(\w+)\}/g, function (v, n) {
        return options[n] || process.env[n] || v;
      });
    } else {
      ret[key] = v;
    }
  }
  return ret;
}

function getRNVersion() {
  const version = JSON.parse(_fsExtra2.default.readFileSync(require.resolve('react-native/package.json', {
    paths: [process.cwd()]
  }))).version;

  // We only care about major and minor version.
  const match = /^(\d+)\.(\d+)\./.exec(version);
  return {
    version,
    major: match[1] | 0,
    minor: match[2] | 0
  };
}

async function getApkInfo(fn) {
  const appInfoParser = new AppInfoParser(fn);
  const bundleFile = await appInfoParser.parser.getEntry(/assets\/index.android.bundle/);
  if (!bundleFile) {
    throw new Error('找不到bundle文件。请确保此apk为release版本，且bundle文件名为默认的index.android.bundle');
  }
  const updateJsonFile = await appInfoParser.parser.getEntry(/res\/raw\/update.json/);
  let appCredential = {};
  if (updateJsonFile) {
    appCredential = JSON.parse(updateJsonFile.toString()).android;
  }
  const { versionName, application } = await appInfoParser.parse();
  let buildTime = 0;
  if (Array.isArray(application.metaData)) {
    for (const meta of application.metaData) {
      if (meta.name === 'pushy_build_time') {
        buildTime = meta.value[0];
      }
    }
  }
  if (buildTime == 0) {
    throw new Error('无法获取此包的编译时间戳。请更新react-native-update到最新版本后重新打包上传。');
  }
  return _extends({ versionName, buildTime }, appCredential);
}

async function getIpaInfo(fn) {
  const appInfoParser = new AppInfoParser(fn);
  const bundleFile = await appInfoParser.parser.getEntry(/payload\/.+?\.app\/main.jsbundle/);
  if (!bundleFile) {
    throw new Error('找不到bundle文件。请确保此ipa为release版本，且bundle文件名为默认的main.jsbundle');
  }
  const updateJsonFile = await appInfoParser.parser.getEntry(/payload\/.+?\.app\/assets\/update.json/);
  let appCredential = {};
  if (updateJsonFile) {
    appCredential = JSON.parse(updateJsonFile.toString()).ios;
  }
  const { CFBundleShortVersionString: versionName } = await appInfoParser.parse();
  let buildTimeTxtBuffer = await appInfoParser.parser.getEntry(/payload\/.+?\.app\/pushy_build_time.txt/);
  if (!buildTimeTxtBuffer) {
    // Not in root bundle when use `use_frameworks`
    buildTimeTxtBuffer = await appInfoParser.parser.getEntry(/payload\/.+?\.app\/frameworks\/react_native_update.framework\/pushy_build_time.txt/);
  }
  if (!buildTimeTxtBuffer) {
    throw new Error('无法获取此包的编译时间戳。请更新react-native-update到最新版本后重新打包上传。');
  }
  const buildTime = buildTimeTxtBuffer.toString().replace('\n', '');
  return _extends({ versionName, buildTime }, appCredential);
}

const localDir = _path2.default.resolve(_os2.default.homedir(), '.pushy');
_fsExtra2.default.ensureDirSync(localDir);
function saveToLocal(originPath, destName) {
  // TODO
  // const destPath = path.join(localDir, destName);
  // fs.ensureDirSync(path.dirname(destPath));
  // fs.copyFileSync(originPath, destPath);
}

function printVersionCommand() {
  console.log('react-native-update-cli: ' + pkg.version);
  try {
    const PACKAGE_JSON_PATH = require.resolve('react-native-update/package.json', {
      paths: [process.cwd()]
    });
    console.log('react-native-update: ' + require(PACKAGE_JSON_PATH).version);
  } catch (e) {
    console.log('react-native-update: 无法获取版本号，请在项目目录中运行命令');
  }
}

const pricingPageUrl = exports.pricingPageUrl = 'https://pushy.reactnative.cn/pricing.html';